/**********************************************************

This software is part of J.-S. Caux's ABACUS library.

Copyright (c) J.-S. Caux.

-----------------------------------------------------------

File:  ABACUS_Fitting.h

Purpose:  Defines functions for fitting: linear regression etc.

***********************************************************/

#ifndef ABACUS_FITTING_H
#define ABACUS_FITTING_H

namespace ABACUS {

  // Functions in src/FITTING directory
  void covsrt (SQMat_DP& covar, Vect<bool>& ia, const int mfit);
  void lin_reg (Vect_DP x, Vect_DP y, Vect_DP sigma, DP& a, DP& b, DP& chisq);
  void lin_reg (Vect_DP x, Vect_DP y, DP& a, DP& b, DP& chisq);
  void mrqmin (Vect_DP& x, Vect_DP& y, Vect_DP& sig, Vect_DP& a,
	       Vect<bool>& ia, SQMat_DP& covar, SQMat_DP& alpha, DP& chisq,
	       void funcs(const DP, Vect_DP&, DP&, Vect_DP&), DP& alambda);
  void mrqcof (Vect_DP& x, Vect_DP& y, Vect_DP& sig, Vect_DP& a,
	       Vect<bool>& ia, SQMat_DP& alpha, Vect_DP& beta, DP& chisq,
	       void funcs (const DP, Vect_DP&, DP&, Vect_DP&));

  // For interpolating:
  void polint(Vect_DP& xa, Vect_DP& ya, const DP x, DP& y, DP& dy);
  void polint(Vect_CX& xa, Vect_CX& ya, const std::complex<DP> x, std::complex<DP>& y, std::complex<DP>& dy);

} // namespace ABACUS

#endif
