/**********************************************************

This software is part of J.-S. Caux's ABACUS library.

Copyright (c) J.-S. Caux.

-----------------------------------------------------------

File:  LiebLin_DSF_par_Prepare.cc

Purpose:  Parallel version of ABACUS using MPICH.

***********************************************************/

#include "ABACUS.h"
//#include "mpi.h" // not needed for Prepare

using namespace ABACUS;

int main(int argc, char *argv[])
{
  char whichDSF;
  DP c_int, L, kBT;
  int N, iKmin, iKmax, paralevel, nr_processors_at_newlevel;
  DP target_sumrule = 1.0e+6;  // effectively deactivated here
  bool refine = true;  // always true for parallel mode
  char* Ix2filenameprefix;

  if (argc < 10) { // provide some info

    cout << endl << "Welcome to ABACUS\t(copyright J.-S. Caux)." << endl;
    cout << endl << "Usage of LiebLin_DSF_par_Wrapup executable: " << endl;
    cout << endl << "This function wraps up an ABACUS parallel mode run." << endl;
    cout << endl << "Provide the following arguments:" << endl << endl;
    cout << "char whichDSF \t\t Which structure factor should be calculated ?  Options are:  "
      "d for rho rho, g for psi psi{dagger}, o for psi{dagger} psi" << endl;
    cout << "DP c_int \t\t Value of the interaction parameter:  use positive real values only" << endl;
    cout << "DP L \t\t\t Length of the system:  use positive real values only" << endl;
    cout << "int N \t\t\t Number of particles:  use positive integer values only" << endl;
    cout << "char* defaultScanStatename:\t\t file [].Ix2 contains the quantum numbers defining the "
      "AveragingState; used as defaultScanStatename" << endl;
    cout << "int iKmin" << endl << "int iKmax \t\t Min and max momentum integers to scan over:  "
      "recommended values:  -2*N and 2*N" << endl;
    cout << "int paralevel" << endl;
    cout << "rank[i], nr_processors[i] \t rank and nr_processors of each earlier paralevels." << endl;
    cout << "int nr_processors_at_new_level \t for this latest parallelization level." << endl;

    return(0);
  }

  else { // correct nr of arguments
    int n = 1;
    whichDSF = *argv[n++];
    c_int = atof(argv[n++]);
    L = atof(argv[n++]);
    N = atoi(argv[n++]);
    Ix2filenameprefix = argv[n++];
    iKmin = atoi(argv[n++]);
    iKmax = atoi(argv[n++]);
    paralevel = atoi(argv[n++]); // paralevel == 1 means that we have one layer of parallelization, so no previous rank and nr_processors to specify
    if (argc != 10 + 2*(paralevel - 1)) ABACUSerror("Wrong nr of arguments in LiebLin_DSF_par_Prepare.");

    Vect<int> rank_lower_paralevels(paralevel - 1);
    Vect<int> nr_processors_lower_paralevels(paralevel - 1);
    for (int i = 0; i < paralevel - 1; ++i) {
      rank_lower_paralevels[i] = atoi(argv[n++]);
      nr_processors_lower_paralevels[i] = atoi(argv[n++]);
    }
    nr_processors_at_newlevel = atoi(argv[n++]);

    // Read the Ix2 from the file:
    Vect<int> Ix2_input(N);
    ifstream Ix2_input_file;
    stringstream filenamestrstream;
    filenamestrstream << Ix2filenameprefix;
    string defaultScanStatename = filenamestrstream.str();

    // Digest files into a unique one for the latest paralevel:
    Wrapup_Parallel_Scan_LiebLin (whichDSF, c_int, L, N, iKmin, iKmax, kBT, defaultScanStatename,
				  paralevel, rank_lower_paralevels, nr_processors_lower_paralevels,
				  nr_processors_at_newlevel);

  }

  return(0);
}
